/*
 * DOTFrame.java
 *
 * Created on 09.02.2010, 21:30:48
 */

package darkomen.dot;

import darkomen.util.BasicFileFilter;
import darkomen.util.Point;
import darkomen.util.Registry;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.imageio.ImageIO;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

/**
 * Displays a Map and the associated DOT-List
 * @author Gabriel Kind
 */
public class DOTFrame extends javax.swing.JFrame {
    File currentFile = null;

    String gameflow;
    String maps;

    DOTParser dotParser;
  
    /** Creates new form DOTFrame */
    public DOTFrame() {
        gameflow = Registry.readRegistry("HKEY_LOCAL_MACHINE\\SOFTWARE\\Electronic Arts\\Dark Omen\\1.0\\paths", "GAMEFLOW");
        maps     = Registry.readRegistry("HKEY_LOCAL_MACHINE\\SOFTWARE\\Electronic Arts\\Dark Omen\\1.0\\paths", "MAPS");

        if (gameflow == null)
        {
            gameflow = Registry.readRegistry("HKEY_LOCAL_MACHINE\\SOFTWARE\\Wow6432Node\\Electronic Arts\\Dark Omen\\1.0\\paths", "GAMEFLOW");
        }
        
        if (maps == null)
        {
            maps = Registry.readRegistry("HKEY_LOCAL_MACHINE\\SOFTWARE\\Wow6432Node\\Electronic Arts\\Dark Omen\\1.0\\paths", "MAPS");
        }

        initComponents();

        if (maps != null && new File(maps + "\\M1_ENG.BMP").exists())
        {
            try
            {
                mapPanel1.setMap(ImageIO.read(new File(maps + "\\M1_ENG.BMP")));
                if (gameflow != null && new File(gameflow + "\\CH1_ALL.DOT").exists())
                {
                    try {
                        dotParser = new DOTParser(new File(gameflow + "\\CH1_ALL.DOT"));
                        mapPanel1.setParser(dotParser);
                        currentFile = new File(gameflow + "\\CH1_ALL.DOT");
                        setTitle("DOT-Editor - " + currentFile.getPath());
                        createPathTree();
                    }
                    catch (Exception e) {}
                }
            }
            catch (IOException e) {}

        }

        DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();
        renderer.setOpenIcon(null);
        renderer.setLeafIcon(null);
        renderer.setClosedIcon(null);
        pathsTree.setCellRenderer(renderer);
        
        //mapPanel1.setPaths(dotparser.paths);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mapPanel1 = new darkomen.dot.MapPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        pathsTree = new javax.swing.JTree();
        jToolBar1 = new javax.swing.JToolBar();
        coordLabel = new javax.swing.JLabel();
        jMenuBar1 = new javax.swing.JMenuBar();
        jMenu1 = new javax.swing.JMenu();
        loadDotItem = new javax.swing.JMenuItem();
        loadMapItem = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JSeparator();
        saveItem = new javax.swing.JMenuItem();
        saveAsItem = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        exitItem = new javax.swing.JMenuItem();
        jMenu2 = new javax.swing.JMenu();
        viewDrawSimple = new javax.swing.JCheckBoxMenuItem();
        viewDrawCurve = new javax.swing.JCheckBoxMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("DOT-Editor");
        setLocationByPlatform(true);

        mapPanel1.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                mapPanel1MouseClicked(evt);
            }
        });
        mapPanel1.addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
            public void mouseMoved(java.awt.event.MouseEvent evt) {
                mapPanel1MouseMoved(evt);
            }
        });

        javax.swing.GroupLayout mapPanel1Layout = new javax.swing.GroupLayout(mapPanel1);
        mapPanel1.setLayout(mapPanel1Layout);
        mapPanel1Layout.setHorizontalGroup(
            mapPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 634, Short.MAX_VALUE)
        );
        mapPanel1Layout.setVerticalGroup(
            mapPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 476, Short.MAX_VALUE)
        );

        javax.swing.tree.DefaultMutableTreeNode treeNode1 = new javax.swing.tree.DefaultMutableTreeNode("JTree");
        pathsTree.setModel(new javax.swing.tree.DefaultTreeModel(treeNode1));
        pathsTree.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                pathsTreeMouseClicked(evt);
            }
        });
        pathsTree.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
            public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
                pathsTreeValueChanged(evt);
            }
        });
        pathsTree.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                pathsTreeKeyPressed(evt);
            }
        });
        jScrollPane1.setViewportView(pathsTree);

        jToolBar1.setFloatable(false);
        jToolBar1.setRollover(true);

        coordLabel.setText("jLabel1");
        jToolBar1.add(coordLabel);

        jMenu1.setText("File");

        loadDotItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.event.InputEvent.CTRL_MASK));
        loadDotItem.setText("Open DOT-File");
        loadDotItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loadDotItemActionPerformed(evt);
            }
        });
        jMenu1.add(loadDotItem);

        loadMapItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        loadMapItem.setText("Open Map Graphic");
        loadMapItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loadMapItemActionPerformed(evt);
            }
        });
        jMenu1.add(loadMapItem);
        jMenu1.add(jSeparator2);

        saveItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.CTRL_MASK));
        saveItem.setText("Save");
        saveItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveItemActionPerformed(evt);
            }
        });
        jMenu1.add(saveItem);

        saveAsItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        saveAsItem.setText("Save As...");
        saveAsItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveAsItemActionPerformed(evt);
            }
        });
        jMenu1.add(saveAsItem);
        jMenu1.add(jSeparator1);

        exitItem.setText("Exit");
        exitItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitItemActionPerformed(evt);
            }
        });
        jMenu1.add(exitItem);

        jMenuBar1.add(jMenu1);

        jMenu2.setText("View");

        viewDrawSimple.setSelected(true);
        viewDrawSimple.setText("Draw Simple Lines");
        viewDrawSimple.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                viewDrawSimpleActionPerformed(evt);
            }
        });
        jMenu2.add(viewDrawSimple);

        viewDrawCurve.setText("Draw Curved Lines");
        viewDrawCurve.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                viewDrawCurveActionPerformed(evt);
            }
        });
        jMenu2.add(viewDrawCurve);

        jMenuBar1.add(jMenu2);

        setJMenuBar(jMenuBar1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 151, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(mapPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
            .addComponent(jToolBar1, javax.swing.GroupLayout.DEFAULT_SIZE, 792, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(mapPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 476, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jToolBar1, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void exitItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitItemActionPerformed
        dispose();
    }//GEN-LAST:event_exitItemActionPerformed

    private void loadMapItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadMapItemActionPerformed
        JFileChooser chooser = new JFileChooser();

        chooser.setDialogTitle("Load Map Graphic");

        chooser.addChoosableFileFilter(new BasicFileFilter("Dark Omen Map Graphic", "bmp"));

        if (maps != null)
            chooser.setCurrentDirectory(new File(maps));

        int returnVal = chooser.showOpenDialog(this);

        if (returnVal == JFileChooser.APPROVE_OPTION)
        {
            try
            {
                BufferedImage img = ImageIO.read(chooser.getSelectedFile());
                if (img == null)
                {
                    JOptionPane.showMessageDialog(this, "No Image file!", "Error", JOptionPane.ERROR_MESSAGE);
                }
                else
                {
                    mapPanel1.setMap(img);
                }
                
            } catch (IOException e)
            {
                JOptionPane.showMessageDialog(this, "Loading Image failed!", "Error", JOptionPane.ERROR_MESSAGE);
            }
        }
    }//GEN-LAST:event_loadMapItemActionPerformed

    private void loadDotItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadDotItemActionPerformed
        JFileChooser chooser = new JFileChooser();

        chooser.setDialogTitle("Load Dot-File");

        chooser.addChoosableFileFilter(new BasicFileFilter("Dark Omen DOT-File", "dot"));

        if (gameflow != null)
            chooser.setCurrentDirectory(new File(gameflow));

        int returnVal = chooser.showOpenDialog(this);

        if (returnVal == JFileChooser.APPROVE_OPTION)
        {
            try
            {
                dotParser = new DOTParser(chooser.getSelectedFile());
                mapPanel1.setParser(dotParser);
                currentFile = chooser.getSelectedFile();
                setTitle("DOT-Editor - " + chooser.getSelectedFile().getPath());
                createPathTree();
                mapPanel1.repaint();
            } catch (Exception e)
            {
                JOptionPane.showMessageDialog(this, "Loading DOT-File failed!", "Error", JOptionPane.ERROR_MESSAGE);
            }
        }
    }//GEN-LAST:event_loadDotItemActionPerformed

    private void mapPanel1MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_mapPanel1MouseClicked
        if (evt.getButton() == MouseEvent.BUTTON1)
        {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) pathsTree.getSelectionPath().getLastPathComponent();

            if (node != pathsTree.getModel().getRoot() &&
                node.getParent() != pathsTree.getModel().getRoot())
            {
                Point userObj = (Point) node.getUserObject();

                if (evt.getPoint().x < 640 && evt.getPoint().y < 480)
                {
                    userObj.x = evt.getPoint().x;
                    userObj.y = evt.getPoint().y;
                    mapPanel1.repaint();
                }
            }
        }
    }//GEN-LAST:event_mapPanel1MouseClicked

    private void pathsTreeValueChanged(javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_pathsTreeValueChanged
        DefaultMutableTreeNode node = (DefaultMutableTreeNode) evt.getPath().getLastPathComponent();

        if (node == pathsTree.getModel().getRoot())
        {
            mapPanel1.setActivePoint(null);
        }
        else if (node.getParent() == pathsTree.getModel().getRoot())
        {
            mapPanel1.setActivePoint(null);
        }
        else
        {
            Point nodeData = (Point)node.getUserObject();
            mapPanel1.setActivePoint(nodeData);
        }
    }//GEN-LAST:event_pathsTreeValueChanged

    private void mapPanel1MouseMoved(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_mapPanel1MouseMoved
        coordLabel.setText("(" + evt.getPoint().x + ", " + evt.getPoint().y + ")");
    }//GEN-LAST:event_mapPanel1MouseMoved

    private void pathsTreeMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_pathsTreeMouseClicked
        if (evt.getButton() == MouseEvent.BUTTON1 && evt.getClickCount() == 2)
        {
            TreePath tp = pathsTree.getPathForLocation(evt.getX(), evt.getY());

            if (tp != null)
            {


                DefaultMutableTreeNode node = (DefaultMutableTreeNode) tp.getLastPathComponent();

                if (node != pathsTree.getModel().getRoot() &&
                    node.getParent() != pathsTree.getModel().getRoot())
                {
                    Point userObj = (Point) node.getUserObject();

                    String input = (String)JOptionPane.showInputDialog(this, "Enter new Coordinates for: " + node.getUserObject()
                            , "Change Coordinates", -1, null, null, userObj.x + ", " + userObj.y);

                    if (input != null)
                    {
                        if (input.matches("^.*[0-9]+.*,.*[0-9]+.*$"))
                        {
                            Point p = (Point) node.getUserObject();
                            Pattern pattern = Pattern.compile("([0-9]+)");
                            Matcher split = pattern.matcher(input);
                            split.find();
                            int x = Integer.parseInt(input.substring(split.start(), split.end()));
                            split.find();
                            int y = Integer.parseInt(input.substring(split.start(), split.end()));
                            if (x >= 0 && x <= 640 && y >= 0 && y <= 480)
                            {
                                p.x = x; p.y = y;
                                mapPanel1.repaint();
                            }
                            else
                            {
                                JOptionPane.showMessageDialog(this, "Width must be between 0 and 640.\n" +
                                        "Height must be between 0 and 480.", "Invalid Input", JOptionPane.ERROR_MESSAGE);
                            }
                        }
                        else
                        {
                            JOptionPane.showMessageDialog(this, "Please enter 2 Values (delimited by ',')", "Invalid Input", JOptionPane.ERROR_MESSAGE);
                        }
                    }
                }
            }
        }
    }//GEN-LAST:event_pathsTreeMouseClicked

    private void pathsTreeKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_pathsTreeKeyPressed
        if (evt.getKeyCode() == java.awt.event.KeyEvent.VK_ENTER)
        {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) pathsTree.getSelectionPath().getLastPathComponent();

            if (node != pathsTree.getModel().getRoot() &&
                node.getParent() != pathsTree.getModel().getRoot())
            {
                Point userObj = (Point) node.getUserObject();

                String input = (String)JOptionPane.showInputDialog(this, "Enter new Coordinates for: " + node.getUserObject()
                        , "Change Coordinates", -1, null, null, userObj.x + ", " + userObj.y);

                if (input != null)
                {
                    if (input.matches("^.*[0-9]+.*,.*[0-9]+.*$"))
                    {
                        Point p = (Point) node.getUserObject();
                        Pattern pattern = Pattern.compile("([0-9]+)");
                        Matcher split = pattern.matcher(input);
                        split.find();
                        int x = Integer.parseInt(input.substring(split.start(), split.end()));
                        split.find();
                        int y = Integer.parseInt(input.substring(split.start(), split.end()));
                        if (x >= 0 && x <= 640 && y >= 0 && y <= 480)
                        {
                            p.x = x; p.y = y;
                            mapPanel1.repaint();
                        }
                        else
                        {
                            JOptionPane.showMessageDialog(this, "Width must be between 0 and 640.\n" +
                                    "Height must be between 0 and 480.", "Invalid Input", JOptionPane.ERROR_MESSAGE);
                        }
                    }
                    else
                    {
                        JOptionPane.showMessageDialog(this, "Please enter 2 Values (delimited by ',')", "Invalid Input", JOptionPane.ERROR_MESSAGE);
                    }
                }
            }
        }
    }//GEN-LAST:event_pathsTreeKeyPressed

    private void saveItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveItemActionPerformed
        if (currentFile == null) return;

        try
        {
            dotParser.save(currentFile);
        }
        catch (IOException e)
        {
            JOptionPane.showMessageDialog(this, "Saving failed:\n\n" + e.getLocalizedMessage()
                    ,"Saving failed:", JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_saveItemActionPerformed

    private void saveAsItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveAsItemActionPerformed
        if (currentFile == null) return;


        JFileChooser chooser = new JFileChooser();

        chooser.setDialogTitle("Save Dot-File");

        chooser.addChoosableFileFilter(new BasicFileFilter("Dark Omen DOT-File", "dot"));

        if (gameflow != null)
            chooser.setCurrentDirectory(new File(gameflow));

        int returnVal = chooser.showSaveDialog(this);

        if (returnVal == JFileChooser.APPROVE_OPTION)
        {
            try
            {
                dotParser.save(chooser.getSelectedFile());
                currentFile = chooser.getSelectedFile();
                setTitle("DOT-Editor - " + chooser.getSelectedFile().getPath());

            }
            catch (IOException e)
            {
                JOptionPane.showMessageDialog(this, "Saving failed:\n\n" + e.getLocalizedMessage()
                        ,"Saving failed:", JOptionPane.ERROR_MESSAGE);
            }
        }
    }//GEN-LAST:event_saveAsItemActionPerformed

    private void viewDrawSimpleActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_viewDrawSimpleActionPerformed
        mapPanel1.drawSimple(viewDrawSimple.isSelected());
    }//GEN-LAST:event_viewDrawSimpleActionPerformed

    private void viewDrawCurveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_viewDrawCurveActionPerformed
        mapPanel1.drawCurve(viewDrawCurve.isSelected());
    }//GEN-LAST:event_viewDrawCurveActionPerformed

    private void createPathTree()
    {
        DefaultTreeModel model = new DefaultTreeModel((TreeNode)pathsTree.getModel().getRoot());
        DefaultMutableTreeNode root = new DefaultMutableTreeNode("DOT");

        model.setRoot(root);

        for (int i = 0; i < dotParser.getPathCount(); ++i)
        {
            ArrayList<Point> points = dotParser.getPath(i);
            root.insert(new DefaultMutableTreeNode("Path " + i), i);
            DefaultMutableTreeNode child = (DefaultMutableTreeNode)root.getChildAt(i);

            for (int j = 0; j < points.size(); ++j)
            {
                child.insert(new DefaultMutableTreeNode(points.get(j)), j);
            }

        }
        pathsTree.setModel(model);
        pathsTree.setSelectionRow(0);
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel coordLabel;
    private javax.swing.JMenuItem exitItem;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenu jMenu2;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JMenuItem loadDotItem;
    private javax.swing.JMenuItem loadMapItem;
    private darkomen.dot.MapPanel mapPanel1;
    private javax.swing.JTree pathsTree;
    private javax.swing.JMenuItem saveAsItem;
    private javax.swing.JMenuItem saveItem;
    private javax.swing.JCheckBoxMenuItem viewDrawCurve;
    private javax.swing.JCheckBoxMenuItem viewDrawSimple;
    // End of variables declaration//GEN-END:variables

}
