package darkomen.dot;

import darkomen.util.Point;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import javax.swing.UIManager;

/**
 * Parses Dark Omen DOT-Files
 * @author Gabriel Kind
 */
public class DOTParser {
    //private ArrayList<GeneralPath> paths;
    private ArrayList<ArrayList<Point>> paths
            = new ArrayList<ArrayList<Point>>();

    private ByteBuffer file;

    private File dotFile;

    public static void main(String[] args)
    {
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }
        catch (Exception e) {
            // Metal L&F
        }

        new DOTFrame().setVisible(true);
    }

    public DOTParser(File dotfile) throws Exception
    {
        dotFile = dotfile;
        //paths = new ArrayList<GeneralPath>();
        
        try
        {
            FileChannel fc = new FileInputStream(dotFile).getChannel();
            file = ByteBuffer.allocate((int)fc.size());
            fc.read(file);
            fc.close();
        }
        catch (IOException e)
        {
            //e.printStackTrace();
            throw new Exception();
        }

        file.order(ByteOrder.LITTLE_ENDIAN);
        file.rewind();

        if (file.getInt() != 0x57444F54)
            throw new Exception();
        
        file.getInt(); file.getInt();

        int pathnum = file.getInt();
        //System.out.println(pathnum);
        for (int i = 0; i < pathnum; ++i)
        {
            paths.add(generatePath());
            for (int j = 0; j < 11; ++j)
                file.getInt();
        }
    }

    private ArrayList<Point> generatePath()
    {
        int lines = file.getInt();
        //System.out.println(lines);
        //GeneralPath path = new GeneralPath();
        ArrayList<Point> pathCoord =
                new ArrayList<Point>();

        int x1 = file.getInt();
        int y1 = file.getInt();

        pathCoord.add(new Point(x1, y1));
        
        file.getInt();
        file.getInt();

        for (int i = 0; i < lines-1; ++i)
        {

            int x2 = file.getInt();
            int y2 = file.getInt();
            file.getInt();
            file.getInt();
            //System.out.println("(" + x1 + ", " + y1 + ") (" + x2 + ", " + y2 + ")");
            //path.append(new Line2D.Float(x1,y1,x2,y2), true);
            x1 = x2; y1 = y2;

            //System.out.println(x1 + " " + y1);
            pathCoord.add(new Point(x1, y1));
        }
        
        //pathCoords.add(pathCoord);

        return pathCoord;
    }

    /*public ArrayList<GeneralPath> getPaths()
    {
        return paths;
    }*/

    /**
     * Returns number of paths in the DOT-File
     * @return Number of Paths
     */
    public int getPathCount()
    {
        return paths.size();
    }

    /**
     * Returns number of Points of one path
     * @param pathNum Path
     * @return Number of Points of Path
     */
    public int getPathPoints(int pathNum)
    {
        return (pathNum < 0 || pathNum > paths.size()-1?
            null : paths.get(pathNum).size());
    }

    /**
     * Returns an Array with that has all coordinates of a path
     * @param pathNum Path
     * @return Array containing all coords
     */
    public ArrayList<Point> getPath(int pathNum)
    {
        return (pathNum < 0 || pathNum > paths.size()-1?
            null : paths.get(pathNum));
    }

    /**
     * Returns x and y-Coordinates of one Point in a Path
     * @param pathNum Path
     * @param pathPoint Point
     * @return Coordinates of the Point
     */
    public Point getPathPoint(int pathNum, int pathPoint)
    {
        if (pathNum < 0 || pathPoint < 0 || pathNum > paths.size()-1)
        {
            return null;
        }
        else
        {
            if (pathPoint >= paths.get(pathNum).size())
            {
                return null;
            }
            else
            {
                return paths.get(pathNum).get(pathPoint);
            }
        }
    }

    /**
     * Saves the DOT-File
     * @param filename File
     */
    public void save(File filename) throws IOException
    {
        FileChannel fc = new FileOutputStream(filename).getChannel();

        file.order(ByteOrder.LITTLE_ENDIAN);
        file.rewind();

        file.getInt(); file.getInt(); file.getInt();

        

        int pathnum = file.getInt();
        //System.out.println(pathnum);
        for (int i = 0; i < pathnum; ++i)
        {
            int lines = file.getInt();
            //System.out.println(lines);
            //GeneralPath path = new GeneralPath();
            ArrayList<Point> pathCoord = paths.get(i);

            for (int j = 0; j < lines; ++j)
            {
                file.putInt(pathCoord.get(j).x);
                file.putInt(pathCoord.get(j).y);
                file.getInt();
                file.getInt();
            }

            //pathCoords.add(pathCoord);

            for (int j = 0; j < 11; ++j)
                file.getInt();
        }
        file.rewind();
        fc.write(file);
        fc.close();
    }
}
