package darkomen.dot;

import darkomen.util.Point;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;
import java.util.ArrayList;

/**
 *
 * @author Gabriel Kind
 */
public class MapPanel extends javax.swing.JPanel {
    //private ArrayList<GeneralPath> paths;

    private Image img;
    private Point2D activePoint;
    private Color[] colors =
    { Color.RED, Color.YELLOW, Color.GREEN, Color.BLUE,
      Color.CYAN, Color.GRAY, Color.MAGENTA, Color.ORANGE,
      Color.PINK, Color.WHITE };

    // 255 0 0, 255 255 0, 0 255 0, 0 0 255,
    // 0 255 255,

    private DOTParser dotParser;

    private boolean drawSimple = true;
    private boolean drawCurve  = false;

    @Override
    public void paint(Graphics g)
    {
        //super.paint(g);
        Graphics2D g2 = (Graphics2D)g;

        if (img != null)
        {
            g.drawImage(img, 0, 0, null);
        }

        if (dotParser != null)
        {
            for (int i = 0; i < dotParser.getPathCount(); ++i)
            {

                g2.setColor(colors[i%colors.length]);

                ArrayList<Point> path = dotParser.getPath(i);

                if (drawCurve)
                {
                    GeneralPath curve = new GeneralPath();
                    curve.moveTo(path.get(0).x, path.get(0).y);

                    for (int j = 2; j < path.size(); j+=2)
                    {
                        //Point2D lastPoint = curve.getCurrentPoint();
                        curve.quadTo(path.get(j-1).x, path.get(j-1).y,
                                path.get(j).x, path.get(j).y);
                        if (j + 1 == path.size()-1)
                        {
                            curve.lineTo(path.get(j+1).x, path.get(j+1).y);
                        }
                    }

                    g2.draw(curve);
                }

                if (drawSimple)
                {
                    for (int j = 1; j < path.size(); ++j)
                    {
                        g.drawLine(path.get(j-1).x, path.get(j-1).y, path.get(j).x, path.get(j).y);
                    }
                }
            }

            if (activePoint != null)
            {
                g.setColor(Color.WHITE);
                g.drawLine((int)activePoint.getX()-5, (int)activePoint.getY()-5, (int)activePoint.getX()+5, (int)activePoint.getY()+5);
                g.drawLine((int)activePoint.getX()-5, (int)activePoint.getY()+5, (int)activePoint.getX()+5, (int)activePoint.getY()-5);
            }
        }
    }

    public void setParser(DOTParser dotParser)
    {
        this.dotParser = dotParser;
    }

    public void setMap(Image map)
    {
        img = map;
        repaint();
    }

    public void setActivePoint(Point point)
    {
        activePoint = point;
        repaint();
    }

    public void drawSimple(boolean enable)
    {
        drawSimple = enable;
        repaint();
    }

    public void drawCurve(boolean enable)
    {
        drawCurve = enable;
        repaint();
    }
}
