package darkomen.util;

import java.io.File;
import java.util.HashSet;
import javax.swing.filechooser.FileFilter;

/**
 * Ein einfacher Filter für den JFileChooser
 * @author Gabriel Kind
 */
public class BasicFileFilter extends FileFilter {
    /* Erlaubte Dateiendungen */
    private HashSet<String> extensions = new HashSet<String>();

    /* Beschreibugn des Filters */
    private String description = "No Description";

    /**
     * Standardkonstruktor ist gesperrt
     */
    private BasicFileFilter() {}

    /**
     * Erstellt einen neuen Dateifilter
     * @param desc Beschreibung des Filters
     * @param extensions Erlaubte Dateiendungen
     */
    public BasicFileFilter(String desc, String ... extensions)
    {
        description = desc;

        for (String ext : extensions) {
            this.extensions.add(ext);
        }
    }

    /**
     * Fügt nachträglich Dateiendungen hinzu
     * @param extension Zu erlaubende Dateiendung
     */
    public void addExtension(String extension)
    {
        extensions.add(extension);
    }

    /**
     * Ändert die Beschreibung des Filters
     * @param desc Neue Beschreibung
     */
    public void setDescription(String desc)
    {
        description = desc;
    }

    /**
     * Prüft ob eine Dateiendung zugelassen ist
     * @param f Datei
     * @return <tt>true</tt> wenn Dateiendung erlaubt (oder Ordner), <tt>false</tt> sonst
     */
    @Override
    public boolean accept(File f) {
        if (f.isFile())
        {
            // Dateiendung ermitteln
            String ext = null;
            String s = f.getName();
            int i = s.lastIndexOf('.');

            if (i > 0 && i < s.length() - 1) {
                ext = s.substring(i+1).toLowerCase();
            }

            // true wenn Endung enthalten
            return extensions.contains(ext);
        }
        // Verzeichnisse immer erlauben
        else if (f.isDirectory())
        {
            return true;
        }
        return false;
    }

    /**
     * Gibt die Filterbeschreibung zurück
     * @return Beschreibung des Filters
     */
    @Override
    public String getDescription() {
        return description;
    }

}
